﻿#region Header
//
//   Project:           PicFx - Windows Phone 7 image effects app.
//
//   Changed by:        $Author$
//   Changed on:        $Date$
//   Changed in:        $Revision$
//   Project:           $URL$
//   Id:                $Id$
//
//
//   Copyright (c) 2010 Rene Schulte
//
//   This program is open source software. Please read the License.txt.
//
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Microsoft.Devices;
using Microsoft.Phone.Controls;
using PicFx.Effects;
using PicFx.Effects.CompositeEffects;
using PicFx.ViewModels;
using System.Windows.Media.Imaging;
using Microsoft.Phone.Tasks;

namespace PicFx
{
    /// <summary>
    /// Main application page
    /// </summary>
    public partial class MainPage : PhoneApplicationPage
    {
        WriteableBitmap original;
        WriteableBitmap resized;
        bool isInitialized;
        private CameraCaptureTask cameraCaptureTask;
        private PhotoChooserTask photoChooserTask;

        public MainPage()
        {
            InitializeComponent();
        }


        //**CameraProxy**
        Darkthread.CameraProxy cameraProxy;

        private void Initialize()
        {
            // Init tasks
            cameraCaptureTask = new CameraCaptureTask();
            cameraCaptureTask.Completed += PhotoProviderTaskCompleted;

            //**CameraProxy**
            //Create CameraProxy object and shared the same event handler
            //with cameraCaptureTask.Completed
            //PS: 192.168.1.136 is the local IP address of host
            cameraProxy = new Darkthread.CameraProxy(
                "http://192.168.1.136:1688/", this);
            cameraProxy.Completed += PhotoProviderTaskCompleted;

            photoChooserTask = new PhotoChooserTask
            {
                //ShowCamera = true,
                //PixelHeight = 640,
                //PixelWidth = 480
            };
            photoChooserTask.Completed += PhotoProviderTaskCompleted;

            // Disable the camera button if the app runs in the emulator
            // Todo: The BtnCamera reference returns null in the beta
            // BtnCamera.IsEnabled = Microsoft.Devices.Environment.DeviceType != DeviceType.Emulator;
            // That's why we have to use this more hacky trick:
            var buttons = ApplicationBar.Buttons.Cast<Microsoft.Phone.Shell.ApplicationBarIconButton>();
            var btn = buttons.Where(b => b.IconUri.ToString().ToLower().Contains("camera")).FirstOrDefault();
            if (btn != null)
            {
                //**CameraProxy**
                //btn.IsEnabled = Microsoft.Devices.Environment.DeviceType 
                //                != DeviceType.Emulator;
                btn.IsEnabled = true;
            }

            // Load and show the initial image
            if (original == null)
            {
                original = new WriteableBitmap(0, 0).FromResource("data/monkeys.jpg");
            }
            ShowImage(original);
        }

        private WriteableBitmap ApplySelectedEffect(WriteableBitmap bitmap)
        {
            // Apply selected effect
            var result = bitmap;
            if (ListBoxEffects != null)
            {
                var item = ListBoxEffects.SelectedItem as EffectItem;
                if (item != null)
                {
                    result = item.Effect.Process(bitmap);
                }
            }
            return result;
        }

        // Todo: Remove fixed height of Viewport image. 
        // It's needed ATM because the unofficial PivotControl doesn't respect the height of the AppBar.
        private void ResizeAndShowImage(WriteableBitmap bitmap)
        {
            resized = bitmap;

            // Fast and simple resize by using UIElement rendering
            if (Viewport != null)
            {
                Viewport.Source = bitmap;
                resized = new WriteableBitmap(Viewport, null);
            }

            // Apply selected effect and show image
            ApplySelectedEffect(resized);
            ShowImage(resized);
        }

        private void ShowImage(ImageSource result)
        {
            // Show image and scroll to start page if needed
            if (Viewport != null)
            {
                Viewport.Source = result;
                if (PivotCtrl.SelectedIndex != 0)
                {
                    PivotCtrl.ScrollPrev();
                }
            }
        }

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            Initialize();
        }

        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (resized != null)
            {
                // Apply selected effect and show image
                var result = ApplySelectedEffect(resized);
                ShowImage(result);
            }
        }

        private void ApplicationBarIconFolderButton_Click(object sender, EventArgs e)
        {
            photoChooserTask.Show();
        }


        //**CameraProxy** Add a flag to identify if it's in emulator mode
        private bool EmulatorMode
        {
            get
            {
                return Microsoft.Devices.Environment.DeviceType 
                       == DeviceType.Emulator;
            }
        }

        private void ApplicationBarIconCameraButton_Click(object sender, EventArgs e)
        {
            //Trigger CameraCaptureTask or CameraProxy depends on EmulatorMode flag
            if (EmulatorMode)
                cameraProxy.Show();
            else
                cameraCaptureTask.Show();
        }

        private void PhotoProviderTaskCompleted(object sender, PhotoResult e)
        {
            // Load the photo from the task result
            if (e != null && e.TaskResult == TaskResult.OK)
            {
                var bmpi = new BitmapImage();
                //**CameraProxy** add branch code for CameraProxy
                bool extCamera = (EmulatorMode && 
                                  e is Darkthread.PhotoResultHacking);
                if (extCamera)
                    bmpi.SetSource(
                        ((Darkthread.PhotoResultHacking)e).CameraPhotoStream);
                else 
                    bmpi.SetSource(e.ChosenPhoto);
                original = new WriteableBitmap(bmpi);
                //**CameraProxy**
                //Real CameraCaptureTask will terminate the application
                //and cause Loaded event, but CameraProxy won't,
                //so we have to add the logic after camera capture here.
                //Ugly, but seems no better way
                if (extCamera)
                {
                    ShowImage(original);
                    ResizeAndShowImage(original);
                }
            }
        }

        private void ApplicationBarIconSaveButton_Click(object sender, EventArgs e)
        {
            var result = ApplySelectedEffect(original);
            var name = String.Format("PicFx_{0:yyyy-MM-dd_hh-mm-ss-tt}.jpg", DateTime.Now);
            result.SaveToMediaLibrary(name);
        }

        private void PhoneApplicationPage_LayoutUpdated(object sender, EventArgs e)
        {
            // Resize here only for the first time to get the real actual width and height
            if (!isInitialized && Viewport != null && Viewport.ActualWidth > 0)
            {
                ResizeAndShowImage(original);
                isInitialized = true;
            }
        }

        private void PhoneApplicationPage_BackKeyPress(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (ListBoxEffects != null && ListBoxEffects.SelectedItem != null)
            {
                ListBoxEffects.SelectedItem = null;
                e.Cancel = true;
            }
        }
    }
}