﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using DarkHttpServer;
using System.Threading;
using System.IO;
using System.Drawing.Imaging;

namespace WebCamGateway
{
    public partial class ViewFinder : Form
    {
        #region Initialization and termination
        //REF: DirectShow.NET Sample: DxSnap-2008
        private Capture cam;
        IntPtr m_ip = IntPtr.Zero;

        public ViewFinder()
        {
            InitializeComponent();
            lblStatus.Text = HTTP_DOWN;
        }

        private void ViewFindercs_Load(object sender, EventArgs e)
        {
            const int VIDEODEVICE = 0; // zero based index of video capture device to use
            const int VIDEOWIDTH = 640; // Depends on video device caps
            const int VIDEOHEIGHT = 480; // Depends on video device caps
            const int VIDEOBITSPERPIXEL = 24; // BitsPerPixel values determined by device
            try
            {
                cam = new Capture(VIDEODEVICE, VIDEOWIDTH, VIDEOHEIGHT, VIDEOBITSPERPIXEL, pbCameraView);
                this.Text += " - " + cam.DeviceName;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Init Failed: " + ex.Message);
                Dispose();
                Close();
            }
        }
        //Use this function to replace Dispose in ViewFinder.Designer.cs
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);

            if (m_ip != IntPtr.Zero)
            {
                Marshal.FreeCoTaskMem(m_ip);
                m_ip = IntPtr.Zero;
            }

            if (httpServer != null)
                httpServer.Stop();
        }

        private void ViewFindercs_FormClosed(object sender, FormClosedEventArgs e)
        {
            cam.Dispose();
            if (m_ip != IntPtr.Zero)
            {
                Marshal.FreeCoTaskMem(m_ip);
                m_ip = IntPtr.Zero;
            }
        }
        #endregion

        #region HTTP Server
        MicroHttpServer httpServer = null;

        const string HTTP_RUNNING = "Host is running...";
        const string HTTP_DOWN = "Host is down.";
        const string HTTP_PROC = "Camera Request Processing...";

        private void btnHttpServerSwitch_Click(object sender, EventArgs e)
        {
            int port = 1688;
            if (!int.TryParse(txtPort.Text, out port))
            {
                MessageBox.Show("Invalid port number: " + txtPort.Text);
                return;
            }
            if (httpServer == null) //To run!
            {
                httpServer = new DarkHttpServer.MicroHttpServer(
                    1688, TakePicture);
                btnHttpServerSwitch.Text = "Stop";
                txtPort.Enabled = false;
                lblStatus.Text = HTTP_RUNNING;
            }
            else //To stop!
            {
                httpServer.Stop();
                httpServer = null;
                btnHttpServerSwitch.Text = "Run";
                txtPort.Enabled = true;
                lblStatus.Text = HTTP_DOWN;
            }
        }

        #endregion

        #region Take picture
        ManualResetEvent mre = new ManualResetEvent(false);
        Bitmap photo = null;
        private CompactResponse TakePicture(CompactRequest req)
        {
            Invoke((Action)(() => {
                btnShutter.Enabled = true;
                btnShutter.ForeColor = Color.DarkGreen;
                lblStatus.Text = HTTP_PROC;
            }));
            mre.Reset();
            mre.WaitOne();
            Invoke((Action)(() =>
            {
                btnShutter.ForeColor = Color.DarkGray;
                btnShutter.Enabled = false;
                lblStatus.Text = HTTP_RUNNING;
            }));
            if (photo == null)
                return new CompactResponse() {
                    StatusText = CompactResponse.HttpStatus.Http404
                };
            MemoryStream ms = new MemoryStream();
            photo.Save(ms, ImageFormat.Png);
            return new CompactResponse()
            {
                ContentType = "image/jpeg",
                Data = ms.ToArray()
            };
        }
        #endregion

        private void btnShutter_Click(object sender, EventArgs e)
        {
            // Release any previous buffer
            if (m_ip != IntPtr.Zero)
            {
                Marshal.FreeCoTaskMem(m_ip);
                m_ip = IntPtr.Zero;
            }

            // capture image
            m_ip = cam.Click();
            Bitmap b = new Bitmap(cam.Width, cam.Height, cam.Stride, PixelFormat.Format24bppRgb, m_ip);

            // If the image is upsidedown
            b.RotateFlip(RotateFlipType.RotateNoneFlipY);
            photo = b;
            mre.Set();
        }

        private void pbCameraView_Click(object sender, EventArgs e)
        {

        }


    }
}
